@@ -42,7 +42,7 @@ pub fn nil() -> Term { fls() }
 /// # }
 /// ```
 pub fn null() -> Term {
-    abs(app!(Var(1), abs(abs(abs(fls()))), tru()))
+    abs(app!(Var(1), abs!(3, fls()), tru()))
 }
 
 /// Equivalent to `pairs::pair()`; applied to two terms it returns them contained in a Church list.
@@ -147,18 +147,18 @@ pub fn tail() -> Term { snd() }
 pub fn length() -> Term {
     app!(
         z(),
-        abs(abs(abs(app!(
+        abs!(3, app!(
             Var(1),
-            abs(abs(abs(abs(abs(Var(1)))))),
-            abs(abs(Var(2))),
+            abs!(5, Var(1)),
+            abs!(2, Var(2)),
             abs(Var(3)),
             abs(app!(
                 Var(4),
-                abs(abs(app(Var(2), app!(Var(5), Var(2), Var(1))))),
-                app(Var(2), abs(abs(Var(1))))
+                abs!(2, app(Var(2), app!(Var(5), Var(2), Var(1)))),
+                app(Var(2), abs!(2, Var(1)))
             )),
             abs(Var(1))
-        )))),
+        )),
         zero()
     )
 }
@@ -188,22 +188,22 @@ pub fn length() -> Term {
 pub fn reverse() -> Term {
     app!(
         z(),
-        abs(abs(abs(app!(
+        abs!(3, app!(
             Var(1),
-            abs(abs(abs(abs(abs(Var(1)))))),
-            abs(abs(Var(2))),
+            abs!(5, Var(1)),
+            abs!(2, Var(2)),
             abs(Var(3)),
             abs(app!(
                 Var(4),
                 abs(app!(
                     Var(1),
-                    app(Var(3), abs(abs(Var(2)))),
+                    app(Var(3), abs!(2, Var(2))),
                     Var(4)
                 )),
-                app(Var(2), abs(abs(Var(1))))
+                app(Var(2), abs!(2, Var(1)))
             )),
             abs(Var(1))
-        )))),
+        )),
         nil()
     )
 }
@@ -229,9 +229,7 @@ pub fn reverse() -> Term {
 pub fn list() -> Term {
     abs(app!(
         Var(1),
-        abs(abs(abs(
-            app(Var(3), app!(pair(), Var(1), Var(2)))
-        ))),
+        abs!(3, app(Var(3), app!(pair(), Var(1), Var(2)))),
         reverse(),
         nil()
     ))
@@ -260,22 +258,22 @@ pub fn list() -> Term {
 /// ```
 pub fn append() -> Term {
     z().app(
-        abs(abs(abs(app!(
+        abs!(3, app!(
             Var(2),
-            abs(abs(abs(abs(abs(Var(1)))))),
-            abs(abs(Var(2))),
+            abs!(5, Var(1)),
+            abs!(2, Var(2)),
             abs(Var(2)),
-            abs(abs(app!(
+            abs!(2, app!(
                 Var(1),
-                app(Var(4), abs(abs(Var(2)))),
+                app(Var(4), abs!(2, Var(2))),
                 app!(
                     Var(5),
-                    app(Var(4), abs(abs(Var(1)))),
+                    app(Var(4), abs!(2, Var(1))),
                     Var(3)
                 )
-            ))),
+            )),
             Var(1)
-        ))))
+        ))
     )
 }
 
@@ -300,13 +298,11 @@ pub fn append() -> Term {
 /// # }
 /// ```
 pub fn index() -> Term {
-    abs(abs(
-        app!(
-            Var(2),
-            abs(app(Var(1), abs(abs(Var(1))))),
-            Var(1),
-            abs(abs(Var(2)))
-        )
+    abs!(2, app!(
+        Var(2),
+        abs(app(Var(1), abs!(2, Var(1)))),
+        Var(1),
+        abs!(2, Var(2))
     ))
 }
 
@@ -333,25 +329,25 @@ pub fn index() -> Term {
 /// ```
 pub fn map() -> Term {
     z().app(
-        abs(abs(abs(app!(
+        abs!(3, app!(
             Var(1),
-            abs(abs(abs(abs(abs(Var(1)))))),
-            abs(abs(Var(2))),
-            abs(abs(abs(Var(1)))),
-            abs(abs(app!(
+            abs!(5, Var(1)),
+            abs!(2, Var(2)),
+            abs!(3, Var(1)),
+            abs!(2, app!(
                 Var(1),
                 app(
                     Var(4),
-                    app(Var(3), abs(abs(Var(2))))
+                    app(Var(3), abs!(2, Var(2)))
                 ),
                 app!(
                     Var(5),
                     Var(4),
-                    app(Var(3), abs(abs(Var(1))))
+                    app(Var(3), abs!(2, Var(1)))
                 )
-            ))),
+            )),
             abs(Var(1))
-        ))))
+        ))
     )
 }
 
@@ -380,10 +376,10 @@ pub fn map() -> Term {
 /// ```
 pub fn foldl() -> Term {
     z().app(
-        abs(abs(abs(abs(app!(
+        abs!(4, app!(
             Var(1),
-            abs(abs(abs(abs(abs(Var(1)))))),
-            abs(abs(Var(2))),
+            abs!(5, Var(1)),
+            abs!(2, Var(2)),
             abs(Var(3)),
             abs(app!(
                 Var(5),
@@ -391,12 +387,12 @@ pub fn foldl() -> Term {
                 app!(
                     Var(4),
                     Var(3),
-                    app(Var(2), abs(abs(Var(2))))
+                    app(Var(2), abs!(2, Var(2)))
                 ),
-                app(Var(2), abs(abs(Var(1))))
+                app(Var(2), abs!(2, Var(1)))
             )),
             abs(Var(1))
-        )))))
+        ))
     )
 }
 
@@ -424,24 +420,22 @@ pub fn foldl() -> Term {
 /// # }
 /// ```
 pub fn foldr() -> Term {
-    abs(abs(abs(
-        app!(
-            z(),
-            abs(abs(app!(
-                Var(1),
-                abs(abs(abs(abs(abs(Var(1)))))),
-                abs(abs(Var(2))),
-                abs(Var(5)),
-                abs(app!(
-                    Var(6),
-                    app(Var(2), abs(abs(Var(2)))),
-                    app(Var(3), app(Var(2), abs(abs(Var(1)))))
-                )),
-                abs(Var(1))
-            ))),
-            Var(1)
-        )
-    )))
+    abs!(3, app!(
+        z(),
+        abs!(2, app!(
+            Var(1),
+            abs!(5, Var(1)),
+            abs!(2, Var(2)),
+            abs(Var(5)),
+            abs(app!(
+                Var(6),
+                app(Var(2), abs!(2, Var(2))),
+                app(Var(3), app(Var(2), abs!(2, Var(1))))
+            )),
+            abs(Var(1))
+        )),
+        Var(1)
+    ))
 }
 
 /// Applied to a predicate and a Church list it filters the list based on the predicate.
@@ -471,28 +465,28 @@ pub fn foldr() -> Term {
 /// ```
 pub fn filter() -> Term {
     z().app(
-        abs(abs(abs(app!(
+        abs!(3, app!(
             Var(1),
-            abs(abs(abs(abs(abs(Var(1)))))),
-            abs(abs(Var(2))),
-            abs(abs(abs(Var(1)))),
+            abs!(5, Var(1)),
+            abs!(2, Var(2)),
+            abs!(3, Var(1)),
             abs(app!(
                 Var(3),
-                app(Var(2), abs(abs(Var(2)))),
-                abs(abs(app!(
+                app(Var(2), abs!(2, Var(2))),
+                abs!(2, app!(
                     Var(1),
-                    app(Var(4), abs(abs(Var(2)))),
+                    app(Var(4), abs!(2, Var(2))),
                     Var(2)
-                ))),
+                )),
                 abs(Var(1)),
                 app!(
                     Var(4),
                     Var(3),
-                    app(Var(2), abs(abs(Var(1))))
+                    app(Var(2), abs!(2, Var(1)))
                 )
             )),
             abs(Var(1))
-        ))))
+        ))
     )
 }
 
@@ -833,98 +827,3 @@ impl Index<usize> for Term {
         candidate.head_ref().unwrap() // safe - verified as valid by is_list()
     }
 }
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-    use reduction::beta;
-    use reduction::Order::*;
-    use church::numerals::{is_zero, plus, succ};
-
-    #[test]
-    fn list_from_vector() {
-        let list_from_vec = Term::from(vec![1.into(), 1.into(), 0.into()]);
-        let list_pushed = nil()
-            .push(0.into())
-            .and_then(|t| t.push(1.into()))
-            .and_then(|t| t.push(1.into()))
-            .unwrap();
-
-        assert_eq!(list_from_vec, list_pushed);
-    }
-
-    #[test]
-    fn list_length() {
-        let list0 = nil();
-        assert_eq!(list0.len(), Ok(0));
-        let list1 = list0.push(1.into()).unwrap();
-        assert_eq!(list1.len(), Ok(1));
-        let list2 = list1.push(1.into()).unwrap();
-        assert_eq!(list2.len(), Ok(2));
-        let list3 = list2.push(1.into()).unwrap();
-        assert_eq!(list3.len(), Ok(3));
-    }
-
-    #[test]
-    fn list_operations() {
-        let list_354 = Term::from(vec![3.into(), 5.into(), 4.into()]);
-
-        assert!(list_354.is_list());
-
-        assert_eq!(list_354.head_ref(), Ok(&3.into()));
-        assert_eq!(list_354.tail_ref(), Ok(&Term::from(vec![5.into(), 4.into()])));
-
-        assert_eq!(list_354.tail_ref().and_then(|t| t.head_ref()), Ok(&5.into()));
-        assert_eq!(list_354.tail_ref()
-                   .and_then(|t| t.tail_ref())
-                   .and_then(|t| t.head_ref()), Ok(&4.into()));
-
-        let unconsed = list_354.uncons();
-        assert_eq!(unconsed, Ok((3.into(), Term::from(vec![5.into(), 4.into()]))));
-    }
-
-    #[test]
-    fn iterating_list() {
-        let list010 = Term::from(vec![0.into(), 1.into(), 0.into()]);
-        let mut iter = list010.into_iter();
-
-        assert_eq!(iter.next(), Some(0.into()));
-        assert_eq!(iter.next(), Some(1.into()));
-        assert_eq!(iter.next(), Some(0.into()));
-        assert_eq!(iter.next(), None);
-    }
-
-    #[test]
-    fn indexing_list() {
-        let list = Term::from(vec![0.into(), 1.into(), 2.into(), 3.into(), 4.into()]);
-
-        assert_eq!(list[0], 0.into());
-        assert_eq!(list[1], 1.into());
-        assert_eq!(list[2], 2.into());
-        assert_eq!(list[3], 3.into());
-        assert_eq!(list[4], 4.into());
-    }
-
-    #[test]
-    fn cbv_list_functions() {
-        let l = Term::from(vec![1.into(), 2.into(), 3.into(), 4.into()]);
-
-        assert_eq!(beta(app!( length(), l.clone()), HAP, 0, false), 4.into());
-
-        assert_eq!(beta(app!(reverse(), l.clone()), HAP, 0, false),
-            Term::from(vec![4.into(), 3.into(), 2.into(), 1.into()]));
-
-        assert_eq!(beta(app!(list(), 4.into(), 1.into(), 2.into(), 3.into(), 4.into()), HAP, 0, false), l);
-
-        assert_eq!(beta(app!(append(), Term::from(vec![1.into(), 2.into()]),
-            Term::from(vec![3.into(), 4.into()])), HAP, 0, false), l);
-
-        assert_eq!(beta(app!(map(), succ(), l.clone()), HAP, 0, false),
-            Term::from(vec![2.into(), 3.into(), 4.into(), 5.into()]));
-
-        assert_eq!(beta(app!(foldl(), plus(), 0.into(), l.clone()), HAP, 0, false), 10.into());
-        assert_eq!(beta(app!(foldr(), plus(), 0.into(), l.clone()), HAP, 0, false), 10.into());
-
-        assert_eq!(beta(app!(filter(), is_zero(), l.clone()), HAP, 0, false), Term::from(vec![]));
-    }
-}
