@@ -408,6 +408,32 @@ macro_rules! app {
     };
 }
 
+/// A macro for multiple abstraction of `Term`s.
+///
+/// # Example
+/// ```
+/// # #[macro_use] extern crate lambda_calculus;
+/// # fn main() {
+/// use lambda_calculus::term::*;
+///
+/// assert_eq!(abs!(3, Var(1)), abs(abs(abs(Var(1)))));
+/// # }
+/// ```
+#[macro_export]
+macro_rules! abs {
+    ($n:expr, $term:expr) => {
+        {
+            let mut term = $term;
+            
+            for _ in 0..$n {
+                term = abs(term);
+            }
+            
+            term
+        }
+    };
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -418,25 +444,36 @@ mod tests {
                    Var(4).app(Var(1).app(Var(2)).app(Var(3)))
         );
     }
+    
+    #[test]
+    fn abs_macro() {
+        assert_eq!(abs!(4, Var(1)),
+                   abs(abs(abs(abs(Var(1)))))
+        );
+        
+        assert_eq!(abs!(2, app(Var(1), Var(2))),
+                   abs(abs(app(Var(1), Var(2))))
+        );
+    }
 
     #[test]
     fn open_term_display() {
         assert_eq!(&format!("{}",     abs(Var(2))) , "λa.b");
         assert_eq!(&format!("{}",     abs(Var(3))) , "λa.c");
-        assert_eq!(&format!("{}", abs(abs(Var(3)))), "λa.λb.c");
-        assert_eq!(&format!("{}", abs(abs(Var(4)))), "λa.λb.d");
+        assert_eq!(&format!("{}", abs!(2, Var(3))), "λa.λb.c");
+        assert_eq!(&format!("{}", abs!(2, Var(4))), "λa.λb.d");
     }
 
     #[test]
     fn display_modes() {
-        let zero = abs(abs(Var(1)));
-        let succ = abs(abs(abs(app(Var(2), app!(Var(3), Var(2), Var(1))))));
-        let pred = abs(abs(abs(app!(
+        let zero = abs!(2, Var(1));
+        let succ = abs!(3, app(Var(2), app!(Var(3), Var(2), Var(1))));
+        let pred = abs!(3, app!(
             Var(3),
-            abs(abs(app(Var(1), app(Var(2), Var(4))))),
+            abs!(2, app(Var(1), app(Var(2), Var(4)))),
             abs(Var(2)),
             abs(Var(1))
-        ))));
+        ));
 
         assert_eq!(&format!("{}", zero), "λa.λb.b");
         assert_eq!(&format!("{}", succ), "λa.λb.λc.b (a b c)");
